#include <curl/curl.h>
#include <sstream>
#include <string>
#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"
using namespace std;

struct Input{
    vector<double> numbers;
    size_t bin_count{};
};

Input input_data(istream& in, bool promt)
{
    if (promt){
        size_t number_count;
        cerr << "Enter number count: ";
        in >> number_count;

        Input In;

        In.numbers.resize(number_count);
        cerr << "Enter numbers: ";
        for(size_t i = 0; i < number_count; i++){
            in >> In.numbers[i];
        }

        cerr << "Enter bin count: ";
        in >> In.bin_count;
        return In;
    }
    else{
        size_t number_count;
        in >> number_count;

        Input In;

        In.numbers.resize(number_count);
        for(size_t i = 0; i < number_count; i++){
            in >> In.numbers[i];
        }

        in >> In.bin_count;
        return In;
    }
}

size_t write_data(void* items, size_t item_size, size_t item_count, void* ctx)
{
    size_t data_size = item_size*item_count;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    buffer->write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

Input download(const string& address)
{
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if(curl) {
        CURLcode res,res1;
        curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        res = curl_easy_perform(curl);
        if (!res){
            curl_off_t speed;
            res1 = curl_easy_getinfo(curl, CURLINFO_SPEED_DOWNLOAD_T, &speed);
            if(!res1){
                cout << "Upload speed bytes/sec\n" << speed;
                exit(1);
            }
        }
        curl_easy_cleanup(curl);
    }
    return input_data(buffer,false);
}


int main(int argc, char* argv[])
{
    Input input;
    if (argc > 1){
        input = download(argv[1]);
    }
    else{
        input = input_data(cin,true);
    }
    const auto bins = make_histogram(input.numbers, input.bin_count);
    show_histogram_svg(bins);
    return 0;
}
